# Copyright 2018 - 2023 Zuse Institute Berlin
# Modified by Karlsruhe Institute of Technology <tomas.stary@kit.edu> in year 2023-2024

# -----------------------------------------------------------------------------
# Configuration section
# You will probably need to set paths and package sources here.
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Installation target
# This specifies the path where to install the third party libraries. You need to
# have write access to that directory. ON INSTALLATION, PREVIOUSLY EXISTING
# CONTENTS IN THAT DIRECTORY WILL BE DELETED.
TARGET=/home/weiser/scratch/target
# TARGET=/scratch/local/bzfweise/Kaskade7.6Dependencies-13.1/installed

# -----------------------------------------------------------------------------
# Download directory
# If the files to download are already available there, they'll not be fetched again.
# DOWNLOAD=/data/numerik/software/KaskadeDependencies/Kaskade7.6Dependencies-13.1/download
# DOWNLOAD=/scratch/local/bzfweise/Kaskade7.6Dependencies-13.1/download
DOWNLOAD=/home/weiser/scratch/download

# -----------------------------------------------------------------------------
# Scratch space
# In this directory, temporary builds may be stored. AFTER INSTALLATION, THIS WILL BE DELETED.
SCRATCH=/home/weiser/scratch/scratch


# -----------------------------------------------------------------------------
# GCC
# Kaskade 7 requires a decent C++ compiler. GCC 8.x is known to work,
# but newer versions or other compilers may work as well. This installs
# GCC from source as specified in the following lines. It may be easier and
# quicker to rely on the compiler that comes with your distribution.

# The location from where to fetch the GCC source. If empty, GCC is not installed
# and the standard C++ compiler "c++" is used. The downloaded file shall be
# a gzip-ed tar archive.
# GCC_SRC_PATH=
GCC_SRC_PATH=ftp://ftp.fu-berlin.de/unix/languages/gcc/releases/gcc-14.1.0/gcc-14.1.0.tar.xz

# The location of the MPC library required by GCC. If your distribution has MPC already
# installed, you may set this as an empty string. The downloaded file shall be
# a gzip-ed tar archive.
MPC_SRC_PATH=https://ftp.gnu.org/gnu/mpc/mpc-1.3.1.tar.gz

# The location of the GMP library required by GCC. If your distribution has GMP already
# installed, you may set this as an empty string.
GMP_SRC_PATH=https://gmplib.org/download/gmp/gmp-6.3.0.tar.xz

# The location of the MPFR library required by GCC. If your distribution has MPFR already
# installed, you may set this as an empty string.
MPFR_SRC_PATH=https://www.mpfr.org/mpfr-4.2.1/mpfr-4.2.1.tar.xz

# -----------------------------------------------------------------------------
# boost
BOOST_SRC_PATH=https://boostorg.jfrog.io/artifactory/main/release/1.85.0/source/boost_1_85_0.tar.bz2

# -----------------------------------------------------------------------------
# BLAS & LAPACK
# Right now, installation of BLAS and LAPACK is not supported here. Viable options are
# Atlas or MKL. Make the libraries to be used known here, e.g., for SuiteSparse.
# Use BLASLIB="-L/path/to/library/directory/ -lblas" and LAPACKLIB="-L/path/to/lib/ -llapack".

# Example: Intel MKL
# Leave MKL_ROOT empty if you don't want to use MKL
# MKL_ROOT=/data/numerik/software/KaskadeDependencies/MKL2023
MKL_ROOT=

# These are required for SuiteSparse make finding MKL
# BLASLIBDIR=$MKL_ROOT
# BLASLIB="-L$BLASLIBDIR -lmkl_intel_lp64 -lmkl_sequential -lmkl_core -Wl,-rpath=$BLASLIBDIR"
# LAPACKLIB=$BLASLIB
BLASLIBDIR=/usr/lib/x86_64-linux-gnu/blas/
LAPACKLIBDIR=/usr/lib/x86_64-linux-gnu/lapack/
BLASLIB="-L$BLASLIBDIR -lblas"
LAPACKLIB="-L$LAPACKLIBDIR -llapack -llapacke"



# Example: Atlas
# Leave ATLAS_SRC_PATH empty if you don't want to use Atlas
ATLAS_SRC_PATH=

# -----------------------------------------------------------------------------
# SuiteSparse with UMFPACK direct solver and including Metis graph partitioning
SUITESPARSE_SRC_PATH=https://people.engr.tamu.edu/davis/SuiteSparse/SuiteSparse-5.4.0.tar.gz

# -----------------------------------------------------------------------------
# MUMPS - direct multifrontal solver 
# This uses Metis and is therefore installed *after* SuiteSparse
MUMPS_SRC_PATH=https://mumps-solver.org/MUMPS_5.6.0.tar.gz

# -----------------------------------------------------------------------------
# ITSOL - iterative solvers 
ITSOL_SRC_PATH=http://www-users.cs.umn.edu/~saad/software/ITSOL/ITSOL_2.tar.gz

# -----------------------------------------------------------------------------
# Hypre - preconditioners
HYPRE_SRC_PATH=https://github.com/hypre-space/hypre/archive/refs/tags/v2.28.0.tar.gz

# -----------------------------------------------------------------------------
# libamiramesh
# This is a library that is not publically available.
LIBAMIRAMESH_SRC_PATH=file:///data/numerik/software/KaskadeDependencies/libamiramesh.tar.gz

# -----------------------------------------------------------------------------
# DUNE
# Kaskade 7 uses Dune, currently in version 2.7.

# The version of DUNE packages
DUNE_VERSION=2.9.1

# The locations from which to download the required DUNE modules.
DUNE_SRC_PATH=https://dune-project.org/download/$DUNE_VERSION

# Some grid modules, in particular alugrid and uggrid, do no longer offer packages for
# download. Thus, we check out the release branches from the git repository.
DUNE_BRANCH=2.9

# -----------------------------------------------------------------------------
# Florian's FP library
# Give the path to the git repository
FP_SRC_PATH=https://github.com/flwende/fp.git

