#! /bin/sh
# Copyright 2018 - 2018 Zuse Institute Berlin
# Modified by Karlsruhe Institute of Technology <tomas.stary@kit.edu> in year 2023-2024
set -e

#------------------------------------------------------------------------------
# Read the necessary paths and locations
. $ENVFILE

if [ -z "$DOWNLOAD" ]; then
  echo ERROR: EMPTY DOWNLOAD DIRECTORY
  exit 1
fi



cd $SCRATCH

downloadAndUnpack () {
  FROM=$1
  cd $DOWNLOAD
  wget -c $FROM
  cd $SCRATCH
  echo running tar xzf
  tar xvzf $DOWNLOAD/`basename $FROM`
}

# get the DUNE core modules and grid manager modules
downloadAndUnpack $DUNE_SRC_PATH/dune-common-$DUNE_VERSION.tar.gz
downloadAndUnpack $DUNE_SRC_PATH/dune-geometry-$DUNE_VERSION.tar.gz
downloadAndUnpack $DUNE_SRC_PATH/dune-grid-$DUNE_VERSION.tar.gz
downloadAndUnpack $DUNE_SRC_PATH/dune-istl-$DUNE_VERSION.tar.gz
downloadAndUnpack $DUNE_SRC_PATH/dune-uggrid-$DUNE_VERSION.tar.gz
downloadAndUnpack $DUNE_SRC_PATH/dune-alugrid-$DUNE_VERSION.tar.gz

# There is an old bug in multilevellineargeometry.hh that we need to patch out.
patch dune-geometry-$DUNE_VERSION/dune/geometry/multilineargeometry.hh $INSTALLHOME/patches/multilineargeometry.hh.patch

# There appears to be an include missing - apparently triggered by changes in GCC 13. Supposed
# to be fixed in Dune 2.9.1
patch dune-geometry-$DUNE_VERSION/dune/geometry/type.hh $INSTALLHOME/patches/type.hh.patch



# building DUNE
# QUESTION: HOW TO CONVINCE CMAKE TO FIND MKL BLAS/LAPACK LIBRARIES INSTALLED IN A NONSTANDARD DIRECTORY?
#           FOR NOW WE LET DUNE USE THE GENERIC BLAS/LAPACK THAT CMAKE FINDS, AND PROVIDE OUR OWN
#           WHEN LINKING KASKADE EXECUTABLES
# QUESTION: HOW TO DISABLE PYTHON BINDINGS? THERE'S A COMPILE TIME ERROR AT LEAST WITH PYTHON 3.11, AND
#           SINCE WE DON'T MAKE USE OF PYTHON, WE SWITCH IT OFF INSTEAD OF PATCHING THE DUNE SOURCE
echo -n "CMAKE_FLAGS=\"" > opts
echo -n "-DCMAKE_CXX_COMPILER=$CXX -DCMAKE_INSTALL_PREFIX=$TARGET -DUMFPACK_ROOT=$TARGET "  >> opts
echo \" >> opts
echo    "DUNE_ENABLE_PYTHONBINDINGS=\"off\"" >> opts

# DUNE uggrid builds parallel (MPI) version of UG if cmake finds an MPI library. Here we 
# try to convince UG to build a sequential version even if an MPI library is available.
sed -e "s/find_package(MPI)/set(MPI_C_FOUND 0)/g" < dune-uggrid-$DUNE_VERSION/CMakeLists.txt > out
mv out dune-uggrid-$DUNE_VERSION/CMakeLists.txt

# Disabling
for i in common geometry istl grid; do
  sed -e "s/DUNE_ENABLE_PYTHONBINDINGS/0/g" < dune-$i-$DUNE_VERSION/CMakeLists.txt > out
  mv out dune-$i-$DUNE_VERSION/CMakeLists.txt
done

./dune-common-$DUNE_VERSION/bin/dunecontrol --opts=$PWD/opts all
./dune-common-$DUNE_VERSION/bin/dunecontrol --opts=$PWD/opts make install

# The config.h include files are not installed by the DUNE installer script, but we need them in Kaskade.
# (apparently, e.g. for HAVE_LAPACK)
cp dune-common-$DUNE_VERSION/build-cmake/config.h $TARGET/include/dune/common/
cp dune-grid-$DUNE_VERSION/build-cmake/config.h $TARGET/include/dune/grid/





#------------------------------------------------------------------------------
# Enter paths into the Makefile. Note that DUNELIB must come before UGLIB, but UG needs dunegeometry behind it.
echo DUNEINC = -I$TARGET/include -I$TARGET/include/dune       >> $TARGET/Makefile.Local
echo DUNELIB = -L$TARGET/lib -ldunegeometry -ldunegrid -ldunecommon -Wl,-rpath=$TARGET/lib >> $TARGET/Makefile.Local
echo ALUGRIDINC = -I$TARGET/include/dune/alugrid              >> $TARGET/Makefile.Local
echo ALUGRIDLIB = -ldunealugrid                               >> $TARGET/Makefile.Local
echo UGINC = -I$TARGET/include/ug                             >> $TARGET/Makefile.Local
echo UGLIB = -lduneuggrid                       -lmpi         >> $TARGET/Makefile.Local
echo HAVE_UG =                                                >> $TARGET/Makefile.Local

#------------------------------------------------------------------------------
# Clean up
# if [ -n "$SCRATCH" ]; then
#   echo running rm -r "$SCRATCH"/* in a few seconds
#   sleep 20
#   rm -r "$SCRATCH"/*
# fi


