#! /bin/bash
# Copyright 2018 - 2019 Zuse Institute Berlin
# Modified by Karlsruhe Institute of Technology <tomas.stary@kit.edu> in year 2023-2024
set -e

# current path - here we expect to find the installation scripts
INSTALLHOME=$PWD

source $1

# -----------------------------------------------------------------------------
# Execution section
# Conceptually, you should not need to modify anything below. Reality may be
# different, though, in particular in different environments or if package
# versions change, you may need to touch the installation logic below.
# -----------------------------------------------------------------------------


LOGFILE=$TARGET/install.log


# -----------------------------------------------------------------------------
# Create empty target directory
echo clearing target directory
mkdir -p $TARGET
if [ -n "$TARGET" ]; then
  rm -rf $TARGET/*
fi

# do not delete download directory contents, as this is intended to prevent
# unnecessary downloads
mkdir -p $DOWNLOAD

# create scratch directory
echo clearing scratch directory
mkdir -p $SCRATCH
if [ -n "$SCRATCH" ]; then
  rm -rf $SCRATCH/*
fi

# -----------------------------------------------------------------------------
# Write all environment variables set into a file for later reference
export ENVFILE=$TARGET/environment.sh
echo "INSTALLHOME=$INSTALLHOME" >> $ENVFILE
echo "TARGET=$TARGET" >> $ENVFILE
echo "DOWNLOAD=$DOWNLOAD" >> $ENVFILE
echo "SCRATCH=$SCRATCH" >> $ENVFILE
echo "GCC_SRC_PATH=$GCC_SRC_PATH" >> $ENVFILE
echo "MPC_SRC_PATH=$MPC_SRC_PATH" >> $ENVFILE
echo "GMP_SRC_PATH=$GMP_SRC_PATH" >> $ENVFILE
echo "MPFR_SRC_PATH=$MPFR_SRC_PATH" >> $ENVFILE
echo "BOOST_SRC_PATH=$BOOST_SRC_PATH" >> $ENVFILE
echo "MKL_ROOT=$MKL_ROOT" >> $ENVFILE
echo "BLASLIBDIR=$BLASLIBDIR" >> $ENVFILE
echo "BLASLIB=\"$BLASLIB\"" >> $ENVFILE
echo "LAPACKLIB=\"$LAPACKLIB\"" >> $ENVFILE
echo "ATLAS_SRC_PATH=$ATLAS_SRC_PATH" >> $ENVFILE
echo "SUITESPARSE_SRC_PATH=$SUITESPARSE_SRC_PATH" >> $ENVFILE
echo "MUMPS_SRC_PATH=$MUMPS_SRC_PATH" >> $ENVFILE
echo "ITSOL_SRC_PATH=$ITSOL_SRC_PATH" >> $ENVFILE
echo "HYPRE_SRC_PATH=$HYPRE_SRC_PATH" >> $ENVFILE
echo "LIBAMIRAMESH_SRC_PATH=$LIBAMIRAMESH_SRC_PATH" >> $ENVFILE
echo "DUNE_VERSION=$DUNE_VERSION" >> $ENVFILE
echo "DUNE_SRC_PATH=$DUNE_SRC_PATH" >> $ENVFILE
echo "DUNE_BRANCH=$DUNE_BRANCH" >> $ENVFILE
echo "FP_SRC_PATH=$FP_SRC_PATH" >> $ENVFILE
echo "LOGFILE=$LOGFILE" >> $ENVFILE




if [ -n "$CXX" -o -n "$CC" ]; then
  echo Environment variables CC or CXX set:
  echo CC = $CC
  echo CXX = $CXX
  echo Wrongly set compilers can lead to all kinds of strange errors when installing GCC.
  echo Check the paths to make sure they are as intended.
fi

if [ ! -f "$INSTALLHOME/installGCC.sh" ]; then
  echo "Error: installGCC.sh not found in current working directory."
  echo "Please execute the install script from the directory where it is located."
  exit 1
fi

PATH=$TARGET/bin:$PATH
echo "PATH=$PATH" >> $ENVFILE

export LD_RUN_PATH=$LD_RUN_PATH:$TARGET/lib
echo "LD_RUN_PATH=$LD_RUN_PATH" >> $ENVFILE

if ! [ -n "$GCC_SRC_PATH" ]; then
  echo "CXX=`which g++`" >> $ENVFILE
  echo "CC=`which gcc`" >> $ENVFILE
  echo "FC=`which gfortran`" >> $ENVFILE
  echo "FTNLIB='-lgfortran -lpthread'" >> $ENVFILE
  echo CXX = `which g++`       >> $TARGET/Makefile.Local
  echo FTNLIB = '-lgfortran -lpthread' >> $TARGET/Makefile.Local
fi


# make BLAS & LAPACK available
echo BLASLIB = $LAPACKLIB $BLASLIB >> $TARGET/Makefile.Local
