 
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                           */
/*  This file is part of the library KASKADE 7                               */
/*  https://www.zib.de/research/projects/kaskade7-finite-element-toolbox     */
/*                                                                           */
/*  Copyright (C) 2024-2024 Zuse Institute Berlin                            */
/*                                                                           */
/*  KASKADE 7 is distributed under the terms of the ZIB Academic License.    */
/*    see $KASKADE/academic.txt                                              */
/*                                                                           */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "dune/grid/config.h"
#include "dune/grid/uggrid.hh"

#include "fem/matrices.hpp"
#include "fem/spaces.hh"

namespace Kaskade
{
  template <int dim>
  using Space = H1Space<Dune::UGGrid<dim>>;

  template <int dim>
  using Conductivity = Dune::FieldMatrix<double,dim,dim>;

  // Explicit mass matrix instantiation for scalar Lagrange elements in space dimensions 2 - 3
  template NumaBCRSMatrix<Dune::FieldMatrix<double,1,1>>
           massMatrix<Space<2>,double,1,double>(Space<2> const& , double const& );
  template NumaBCRSMatrix<Dune::FieldMatrix<double,1,1>>
           massMatrix<Space<3>,double,1,double>(Space<3> const& , double const& );

  // Explicit stiffness matrix instantiation for scalar Lagrange elements in space dimensions 2 - 3
  template NumaBCRSMatrix<Dune::FieldMatrix<double,1,1>>
           stiffnessMatrix<Space<2>,Conductivity<2>,1,double>(Space<2> const& , Conductivity<2> const& );
  template NumaBCRSMatrix<Dune::FieldMatrix<double,1,1>>
           stiffnessMatrix<Space<3>,Conductivity<3>,1,double>(Space<3> const& , Conductivity<3> const& );


}
