/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                           */
/*  This file is part of the library KASKADE 7                               */
/*  https://www.zib.de/research/projects/kaskade7-finite-element-toolbox     */
/*                                                                           */
/*  Copyright (C) 2002-2023 Zuse Institute Berlin                            */
/*                                                                           */
/*  KASKADE 7 is distributed under the terms of the ZIB Academic License.    */
/*    see $KASKADE/academic.txt                                              */
/*                                                                           */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef IO_TOOLS_HH
#define IO_TOOLS_HH

#if HAVE_UG
#include <dune/grid/uggrid/uggridfactory.hh>
#endif

#if ENABLE_ALUGRID
#include <dune/alugrid/3d/gridfactory.hh>
#endif

namespace Kaskade
{
  namespace IOTools
  {
    /**
     * \cond internals
     */

    /// Wrapper hiding the differences in the grids constructors;
    /// This is obsolete as of Dune 2.9.0
    template <class Grid>
    struct FactoryGenerator
    {
      using Factory = Dune::GridFactory<Grid>;

      static Factory createFactory(unsigned int /* unused */ =0)
      {
        return Factory();
      }
    };

    template <class Grid>
    struct BoundarySegmentIndexWrapper
    {
      template <class Factory, class VertexVector, class BoundaryVector, class TmpIdVector, class IdVector>
      static void readBoundarySegmentIndices(Grid const& grid, Factory const& factory, VertexVector const& vertices, BoundaryVector const& boundary, TmpIdVector const& boundaryIdsFromFile, IdVector& boundaryIds)
      {
        boundaryIds.resize(boundaryIdsFromFile.size());
        for (auto const& cell : elements(grid.leafGridView()))
          for (auto const& intersection : intersections(grid.leafGridView(), cell))
            if (intersection.boundary())
              boundaryIds[intersection.boundarySegmentIndex()]
              = boundaryIdsFromFile[factory.insertionIndex(intersection)][0];
      }
    };

    /**
     * \endcond
     */
  } // namespace IOTools
} // namespace Kaskade

#endif // IO_TOOLS_HH
