/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                           */
/*  This file is part of the library KASKADE 7                               */
/*  https://www.zib.de/research/projects/kaskade7-finite-element-toolbox     */
/*                                                                           */
/*  Copyright (C) 2002-2019 Zuse Institute Berlin                            */
/*                                                                           */
/*  KASKADE 7 is distributed under the terms of the ZIB Academic License.    */
/*    see $KASKADE/academic.txt                                              */
/*                                                                           */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef READPOLY_HH
#define READPOLY_HH

#include <memory>
#include <string>

#include <dune/grid/config.h>

#include "dune/grid/uggrid.hh"

/**
 * \ingroup gridInput
 * \brief Construct an unstructured UG grid from .node and .ele files (Triangle output).
 *
 * \param name the base name of the files (without the .node/.ele extension).
 * \param boundaryMarker a pointer to a vector of boundary markers. If not the nullpointer, the given vector is
 *                       resized to the number of boundary segments (which is the number of boundary edges),
 *                       and the corresponding markers are stored.
 *                       These are stored in arbitrary order.
 * 
 * If the reading of boundary markers is requested, i.e. `boundaryMarker != nullptr`, a ".edge" file must exist.
 * This can be generated by triangle using the -e option.
 *
 * \warning: triangle assigns the "boundary marker" 0 to interior edges in the *.edge file. These are
 *           ignored when reading the boundary information. Thus, don't ever use boundary marker 0
 *           for any boundaries!
 *
 *
 * Dune boundary intersections have a method `boundarySegmentIndex()` returning the index of the
 * coarse grid boundary segment on which the intersection resides. Use this index to look up the marker
 * in the `boundaryMarker` array.
 */
std::unique_ptr<Dune::UGGrid<2>> readPolyData(std::string const& name, std::vector<int>* boundaryMarker=nullptr);

#endif
