/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                           */
/*  This file is part of the library KASKADE 7                               */
/*  https://www.zib.de/research/projects/kaskade7-finite-element-toolbox     */
/*                                                                           */
/*  Copyright (C) 2002-2024 Zuse Institute Berlin                            */
/*                                                                           */
/*  KASKADE 7 is distributed under the terms of the ZIB Academic License.    */
/*    see $KASKADE/academic.txt                                              */
/*                                                                           */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include "umfpack.h"

#include "linalg/factorization.hh"

#include <cassert>
#include <type_traits>

namespace Kaskade
{

  template <class Scalar, class SparseIndexInt>
  void tripletToCompressedColumn(SparseIndexInt nRows, SparseIndexInt nCols,
                                 size_t nNonZeros,
                                 std::vector<SparseIndexInt> const& ridx,
                                 std::vector<SparseIndexInt> const& cidx,
                                 std::vector<Scalar> const& values,
                                 std::vector<SparseIndexInt>& Ap, std::vector<SparseIndexInt>& Ai,
                                 std::vector<Scalar>& Az)
  {
    assert(cidx.size()==nNonZeros &&ridx.size()==nNonZeros && values.size()==nNonZeros);
    Ap.resize(nCols+1);
    Ai.resize(nNonZeros);
    Az.resize(nNonZeros);
    SparseIndexInt status;

    if constexpr (std::is_same_v<int,SparseIndexInt>)
    {
      status = umfpack_di_triplet_to_col(nRows, nCols, nNonZeros,
                                        &ridx[0], &cidx[0], &values[0],
                                        &Ap[0], &Ai[0], &Az[0], nullptr);
      umfpack_di_report_status(0,status);
    }

    if constexpr (std::is_same_v<long,SparseIndexInt>)
    {
      status = umfpack_dl_triplet_to_col(nRows, nCols, nNonZeros,
                                        &ridx[0], &cidx[0], &values[0],
                                        &Ap[0], &Ai[0], &Az[0], nullptr);
      umfpack_dl_report_status(0,status);
    }
    assert(status==UMFPACK_OK);
  }

  // explicit instantiation
  template void tripletToCompressedColumn(int, int, size_t,
                                          std::vector<int> const&, std::vector<int> const&,
                                          std::vector<double> const&,
                                          std::vector<int>&, std::vector<int>&,
                                          std::vector<double>& );

  template void tripletToCompressedColumn(long, long, size_t,
                                          std::vector<long> const&, std::vector<long> const&,
                                          std::vector<double> const&,
                                          std::vector<long>&, std::vector<long>&,
                                          std::vector<double>& );
  
}  // namespace Kaskade
