

% Octave script for generating C++ code for optimized SDC iteration matrices
% Execute it in this directory.

% This file will be included into sdc.cpp within an anonymous namespace
% in the Kaskade namespace.
out = fopen('../sdcMatrixData.cpp','w');

% First create single Euler matrix data for Radau grids up to 6 collocation points (order 11).
% We focus on worst case contraction (which for this setup tends to be at large negative
% eigenvalues lambda).
n = 6;
s = zeros(n,1);
for i=1:n
  s(i) = singleEulerOptimization(radau(i),2*i-1,inf);
end
fprintf(out,'// coefficient for single step size Euler sweeps on Radau grids\n');
fprintf(out,'std::array<double,%d> singleEulerRadauData{ %f',n,s(1));
fprintf(out,', %f',s(2:n));
fprintf(out,'};\n\n');

% Next create SDIRK matrix data for Radau grids up to 6 collocation points (order 11).
% We focus on worst case contraction (which for this setup tends to be at large negative
% eigenvalues lambda).
n = 6;
fprintf(out,'// hatS matrix entries for SDIRK sweeps on Radau grids (in LAPACK column major storage)\n');
fprintf(out,'std::array<std::vector<double>,%d> SDIRKRadauData{\n',n);
for i=1:n
  hatS = SDIRKOptimization(radau(i),2*i-1,inf);
  hatS = hatS(2:end,:); % in Kaskade, we don't store the zero first row of S or hatS
  fprintf(out,'  std::vector<double>{%f',hatS(1,1));
  fprintf(out,', %f',hatS(:)(2:end));
  if i<n
    fprintf(out,'},\n');
  else
    fprintf(out,'}\n');
  end
end
fprintf(out,'};\n\n');

fclose(out);
