% This computes a n x n spectral quadrature matrix S, such that
% S_{ik} =  \int_{\tau = t_{i-1}}^{t_i} L_k(\tau) d\tau
% for i=0,...,n ,k=0,...,n with t_0 = 0 and L_0 = 0

% t: n collocation points t_1, ... , t_n in [0,1] (row vector)
function S = integrationMatrix(t)

if t(1)==0
  printf("*** not yet supported: Lobatto ***\n");
end


n = length(t);
S = zeros(n+1,n+1);

tau = [0 t];
dt = tau(2:end)-tau(1:end-1);

for k=1:n
  % Obtain the Lagrange interpolation polynomial for node k
  y = zeros(1,n);
  y(k) = 1;
  Lk = polyfit(t,y,n-1);

  % and integrate and evaluate it
  iLk = polyint(Lk);
  iLkval = polyval(iLk,tau);

  % compute the integrals
  S(:,k+1) = [0 (iLkval(2:end) - iLkval(1:end-1))]';
end




