 


function t = radau(n)
% The interior Radau points on ]-1,1[ are the zeros of the Jacobi polynomials P^(1,0) and are obtained as
% eigenvalues of the symmetric tridiagonal matrix with A_00 = -1/3, A_jj = -1/(2j+1)/(2j+3), A_j(j-1) = 2j(j+1)/(2j+1)/sqrt(2j(2j+2))
% See Deuflhard/Bornemann 6.3.2 and Wikipedia on Jacobi polynomials.

if n==1
  t = 1;
  return;
end

A = zeros(n-1,n-1);
A(1,1) = -1/3;
for j=1:n-2
  A(j+1,j+1) = -1/(2*j+1)/(2*j+3);
  A(j+1,j) = 2*j*(j+1)/(2*j+1)/sqrt(2*j*(2*j+2));
  A(j,j+1) = A(j+1,j);
end
t =  [sort(eig(A))' 1]/2 + 0.5;

