# Octave script for creating a triangle input file in .poly format
# This creates a rectangular domain [0,w] x [0,1] with a circular
# hole of radius r centered at (0.5,0.5).
# This is a classical test case for instationary Navier-Stokes,
# for developing a Karman vortex street.
#
# Parameters:
# w: width of the domain
# n: number of circumferential points
# r: obstacle radius
#
# A reasonable grid is obtained by
# karman(5,80,0.2)
# triangle -q -a0.001 karman.poly
function karman(w,n,r)

fid = fopen('karman.poly','w')

fprintf(fid,'# CFD domain for Karman vortex shedding\n');

fprintf(fid,'# Vertices\n');
fprintf(fid,'%d 2 0 0\n',n+4);
fprintf(fid,'0 0 0\n');
fprintf(fid,'1 %f 0\n',w);
fprintf(fid,'2 %f 1\n',w);
fprintf(fid,'3 0 1\n');
for i=0:n-1
  fprintf(fid,'%d %f %f\n',4+i,0.5+r*cos(2*pi*i/n),0.5+r*sin(2*pi*i/n));
end

fprintf(fid,'# Edges\n');
fprintf(fid,'%d 0\n',n+4);
fprintf(fid,'0 0 1\n');
fprintf(fid,'1 1 2\n');
fprintf(fid,'2 2 3\n');
fprintf(fid,'3 3 0\n');
for i=0:n-1
  fprintf(fid,'%d %d %d\n',4+i,4+i,mod(i+1,n)+4);
end

fprintf(fid,'# Holes\n');
fprintf(fid,'1\n');
fprintf(fid,'0 0.5 0.5\n');
