/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                           */
/*  This file is part of the library KASKADE 7                               */
/*  https://www.zib.de/research/projects/kaskade7-finite-element-toolbox     */
/*                                                                           */
/*  Copyright (C) 2024-2024 Zuse Institute Berlin                            */
/*                                                                           */
/*  KASKADE 7 is distributed under the terms of the ZIB Academic License.    */
/*    see $KASKADE/academic.txt                                              */
/*                                                                           */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "utilities/logging.hh"

#include <cassert>

namespace Kaskade
{
  Logger::Logger(int lvl, std::ostream& o)
  : level(lvl)
  , out(&o)
  , discard("/dev/null")
  {
  }

  Logger::~Logger()
  {
    out->flush();
  }

  bool Logger::operator()(int lvl, std::string const& message)
  {
    (*this)(lvl) << message;
    return lvl <= level;
  }

  std::ostream& Logger::operator()(int lvl)
  {
    if (lvl <= level)
      return *out;
    else
      return discard;
  }

}
