/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                           */
/*  This file is part of the library KASKADE 7                               */
/*  https://www.zib.de/research/projects/kaskade7-finite-element-toolbox     */
/*                                                                           */
/*  Copyright (C) 2024-2024 Zuse Institute Berlin                            */
/*                                                                           */
/*  KASKADE 7 is distributed under the terms of the ZIB Academic License.    */
/*    see $KASKADE/academic.txt                                              */
/*                                                                           */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef LOGGING_HH
#define LOGGING_HH


#include <fstream>
#include <iostream>


namespace Kaskade
{
  /**
   * \ingroup io
   * \brief Simple logging facility
   */
  class Logger
  {
  public:
    /**
     * \brief Constructor.
     * \param level all log messages with a level greater than the provided level are discarded
     * \param out the output stream to which the log messages are sent. The stream has to exist
     *        during the lifetime of the logger (a reference is kept internally).
     */
    Logger(int level, std::ostream& out=std::cout);

    ~Logger();

    /**
     * \brief log a message with given level
     */
    bool operator()(int level, std::string const& message);

    /**
     * \brief Provides access to the log output stream used for messages of the given level
     */
    std::ostream& operator()(int level);

  private:
    int level;
    std::ostream* out;
    std::ofstream discard;
  };
}

#endif
